/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/* 
     <APPLET 
	CODEBASE=".."
	CODE="hamlet.AppButton.class" 
	NAME=Rational.Rational
	LABEL=Rational
	WIDTH=120 
	HEIGHT=25
	ALIGN=bottom>
	<param name=LABEL value="Launch Rational">
	
	If you see this, then Java is not running in your browser!<br>
	An applet would normally go here...<br>

      </APPLET>
*/


/* Written by Elizabeth S. Odekirk */

package Rational;

import hamlet.*;
import java.awt.*;
import java.util.*;
import java.applet.Applet;

public class Rational extends AppFrame implements Runnable {
	

    // -------------------------------------------------------------
    // Instance variables required for all appShells
    //     But, they don't need to go by these names.	
    // -------------------------------------------------------------
    
  Thread 	appThread     = null; /* the application thread	     */

  final public String version  = "v1.0";
    
  /* Screen */
  Panel    enumeration, 
           input, top, bottom, statusLinePanel,
           enumLabels,  enumAndLabel;
  ScrollableNumbers     minInt, maxInt;
  TextArea rationalEnum;
  MyCanvas plot;
  Eqn	   eqn;
  Button   incrPlot, decrPlot;
  Label    statusLine, mantissaEnumLabel, floatEnumLabel;
  MenuBar  menubar;
  Menu     haltMenu;
  MenuItem haltItem;


  double mantarray[] = new double[10000];
  public double numbers[] = new double[100000];
  int counter = 0;
  boolean firstTime = true;
  double peramXMin;
  RatSharedInfo sharedInfo = new RatSharedInfo();

  int defCursor;
  Cursor curCursor = new Cursor();

  int enumType = 0; /* 0:   summary enum
		       1:   full enum */
  final int summaryEnum = 0;
  final int fullEnum = 1;

  /* default values */
  final int minIntDef   = -2;
  final int maxIntDef   = 2;
  final double plotStartDef = 0;
  final double plotEndDef   = 10;

  public Rational() 
  {

    /*resize(900,900);*/

    defCursor = getCursorType();
    setLayout(new BorderLayout());  

    setUpHelp();
    setUpScreen();
    appThread = new Thread(this);
    appThread.start();
    requestFocus();
    pack();

  }

    //**************************************************************
    //  Start() runs each time the applet runs
    //**************************************************************/
  public void start()
  {
    appThread = new Thread(this);
    appThread.start();
    /*requestFocus();*/
  }

  public synchronized void stop()
  {
    if (appThread != null) 
      appThread.stop();
    appThread     = null;	// Kill the application thread 	
    setCursor(defCursor);
    curCursor.cursor = defCursor;
  }


    //**************************************************************
    //  appStop() runs each time that the app is told to do this
    //            function via the controller applets.  (ie,
    //            Hamlet messages)
    //**************************************************************/

  public void appStop() {
    if (appThread != null)
      appThread.stop();
    appThread = null;          // Kill the application thread
    /*showStatus("Stop the Applet");*/
  }

  //**************************************************************
  //  appStart() runs each time that the app is told to do this
  //             function via the controller applets. (ie, 
  //             Hamlet messages)
  //**************************************************************/
    
  public void appStart() {
    appThread = null;
    if (appThread == null) 
      appThread = new Thread(this);
    appThread.start();
    requestFocus();
/*    showStatus("Start the Applet");*/
  }


  //**************************************************************
  //  appReset() runs each time that the app is told to do this
  //             function via the controller applets.
  //**************************************************************/

  public void appReset() {
      minInt.setIntText(minIntDef);
      maxInt.setIntText(maxIntDef);
      appThread = new Thread(this);
/*      showStatus("Reset the Applet");*/
  }

  /* This takes a message from a send button (this is
   * in the hamlet package) on the web page, and
   * parses it.  
   *
   * handles messages of the format:
   *   "recalc minint <int>  maxint <int>"
   *   "replot start <double> stop <double>"
   *   "eqn <equation string>"
   *
   * Anything else will give an error on stdout.
   */
  protected void handleMessage (String msg) {
    StringTokenizer st = new StringTokenizer(msg);
    String curToken;
    int minint = -1, maxint = -1;
    int gotArgs = 0;
    int i;
    double minx = 0, maxx = 100;

    /*System.out.println("got a message");*/

    try {
      while (st.hasMoreTokens()) {
	curToken = st.nextToken();
	/*System.out.println("Token: " + curToken);*/
	if (curToken.equalsIgnoreCase("recalc")) {
	  /* need both arguments */
	  gotArgs = 0;
	  for (i=0; i<2; i++) { 
	    curToken = st.nextToken();
	    /*System.out.println("Token: " + curToken);*/
	    if (curToken.equalsIgnoreCase("Minint")) {
	      minint = (Integer.valueOf(st.nextToken())).intValue();
	      /*System.out.println("Minint is: " + minint);*/
	      gotArgs = gotArgs + 1;
	      /*System.out.println("+gotArgs: " + gotArgs);*/
	    }
	    if (curToken.equalsIgnoreCase("Maxint")) {
	      maxint = (Integer.valueOf(st.nextToken())).intValue();
	      /*System.out.println("Maxint is: " + maxint);*/
	      gotArgs = gotArgs + 2;
	      /*System.out.println("#gotArgs: " + gotArgs);*/
	    }
	  /*System.out.println("gotArgs: " + gotArgs);*/
	  } /* end for */
	  /*System.out.println("gotArgs: " + gotArgs);*/
	  if (gotArgs != 3) {
	    System.out.println("Error in message - wrong parameters");
	    System.out.println("Msg: " + msg);
	    return;
	  }
	  minInt.setIntText(minint);
	  maxInt.setIntText(maxint);
	  appStart();
	}  /* End of recalc section */
      if (curToken.equalsIgnoreCase("replot")) {
	for (i = 0; i < 2; i++) {
	  curToken = st.nextToken();
	  if (curToken.equalsIgnoreCase("Start")) {
	    minx = (Double.valueOf(st.nextToken())).doubleValue();
	    plot.setXmin(minx);
	  }
	  if (curToken.equalsIgnoreCase("Stop")) {
	    maxx = (Double.valueOf(st.nextToken())).doubleValue();
	    plot.setXmax(maxx);
	  }
        } /* end of for */
	plot.repaint();
      
      }/* End of replot */

      else if (curToken.equalsIgnoreCase("eqn")) {
	msg = msg.substring(3);
	msg = msg.trim();
	eqn.eqnField.setText(msg);
	eqn.parseIt();
	break;			// break out of the while loop
      } /* End of eqn */
      else {
        System.out.println("Unknown command.  Use Recalc, Replot, or Eqn.");
        System.out.println("Msg: " + msg);
	return;
      }

      } /* End of While */

      } /* End of try */
      catch (NoSuchElementException e) {
        System.out.println("Error in message - format error");
        System.out.println("Msg: " + msg + "; Error token: " 
			   + e.getMessage());
        return;
      }
      catch (NumberFormatException e) {
        System.out.println("Error in message - format error in number:");
        System.out.println("Msg: " + msg);
        System.out.println("Error token: " + e.getMessage());
        return;
      }

  } /* end handleMessage() */

 void setUpHelp() {
    helpText = 
      "This applet graphs and lists the rational numbers\n"		+ 
      "which can be formed using a range between a minimum\n"		+
      "and maximum integer as numerator or denominator.  It\n"		+
      "is based on the discussion in Chapter 2 of\n"			+
      "\"Introduction to Scientific Programming\" by \n"		+
      "Joseph L. Zachary.\n\n"						+ 
      //								//
      " File Menu\n"							+
      "   + Reset:  Returns to the default plot region\n"		+
      "   + Refresh: Refreshes the applet\n"				+
      "   + Hide:  Removes the applet from the display\n"		+
      "   + Quit:  Terminates the applet\n\n"				+
      //								//
      " Halt Menu\n"							+
      "   + Halt: Halts the current recalculation.  This puts the\n"	+
      "           applet into an indeterminant state.\n\n"              +
      //								//
      " Help Menu\n"							+
      "   + With Applet:  Displays this help dialog\n\n"		+
      //								//
      " Fields \n"							+ 
      "   + Minimum Integer: Choose the minimum integer by clicking\n"	+
      "     on the up (>) or down (<) arrow\n"				+
      "   + Maximum Integer: Choose the maximum integer by clicking\n"	+
      "     on the up (>) or down (<) arrow\n"				+
      "\n"								+
      //								//
      " Expression\n"							+
      "   + Input: You can enter an expression here, and see if it\n"	+
      "            could complete with the given constraints on\n"	+
      "            minimum and maximum integer.  You can use +, -,\n"	+
      "            *, /, and ^ (exponentiation), as well as ( and ).\n" +
      "            You can also type in a single number to see if\n"    + 
      "            it is in the number system.\n"			+
      "   + Status: If the expression completes successfully, this will\n"+
      "             say \"Successfully Completed\".  There are two\n"	+
      "             kinds of errors, syntax errors and computation errors.\n"+
      "             A syntax error means that your input is incorrect;\n"+
      "             either one of the numbers you put in is not in the\n"+
      "             current number system, or you typed in an\n"	+
      "             incorrect format, such as two slashes (/) in a \n"	+
      "             row.  A computation error means that your input was\n"+
      "             alright, but one of the interim results in solving\n"+
      "             the expression, or the final result, could not be \n"+
      "             represented.\n\n"					+
      //								//
      " Plot \n"							+
      "   + <<: decreases the maximum of the plot\n"			+ 
      "         by an order of magnitude\n"				+
      "   + >>: increases the maximum of the plot\n"			+
      "         by an order of magnitude\n"				+
      "   + dragging the mouse: \tcreate a zoom-in rectangle\n\n"	+ 
      //								//
      " Displayed Information\n"					+
      "  + Scrolling Field of numbers: List of the rational numbers\n"	+
      "    which can be produced using the range of numbers between \n" +
      "    min and max integer as numerator or denominator\n"		+
      "  + Plot: Plot of the above list of numbers\n"			+
      "\n";
 }

 void setUpScreen() {
   /*      showStatus("Setting up the screen");*/

    setTitle("Rational Numbers");

    setLayout(new BorderLayout(2, 2));

    /* This needs to be here since it is sent (as a decoy) to
       scrollable numbers */
    plot = new MyCanvas((Frame) this, curCursor);
    rationalEnum = new TextArea(5, 25);  

    top = new Panel();
    top.setLayout(new BorderLayout(2,2));

    

    /* Make the area where the Mantissa and exponent are
       specified. */
    input = new Panel();
    input.setLayout(new GridLayout(1,0,2,2));  
      
    /* Set up the input fields */
    maxInt = new ScrollableNumbers (maxIntDef, /* initial value */
				      1, /* small increment */
				      0, /* large increment */
				      -10, /* lower limit */
				      10, /* upper limit */
				      3,    /* number of chars */
				      /* location of label */
				      ScrollableNumbers.TOP,
				      "Maximum Integer", /* label */
				      /* who to notify when this field
					 changes */
				      this
				     );
				      
				      
    minInt = new ScrollableNumbers (minIntDef, /* initial value */
				      1, /* small increment */
				      0, /* large increment */
				      -10, /* lower limit */
				      10, /* upper limit */
				      3,    /* number of chars */
				      /* location of label */
				      ScrollableNumbers.TOP,
				      "Minimum Integer", /* label */
				      /* who to notify when this field
					 changes */
				      this
				     );
				      
				      

    maxInt.setEditable(false);
    minInt.setEditable(false);


    /* Top Label for the mantissa and exp specs */
    Label label;


    input.add(minInt);
    input.add(maxInt);

    eqn = new Eqn(this);

    /* Put everything into the top panel */
    top.add("North", input);
    top.add("South", eqn);
    

    /* **The center piece** */
    rationalEnum.setEditable(false);


    /* **bottom panel** */
    bottom = new Panel();

    bottom.setLayout(new BorderLayout());


    plot.resize(500,100);
    plot.setForeground(Color.cyan);
    plot.setBackground(Color.gray);
    bottom.add("Center", plot);
    incrPlot = new Button (">>");
    bottom.add("East", incrPlot);
    decrPlot = new Button ("<<");
    bottom.add("West", decrPlot);


//    System.out.println("Plot Size is " + plot.size());
    statusLine = new Label("This is the Status Line " + 
			   "	                    ");
    statusLinePanel = new Panel();
    statusLinePanel.add(statusLine);

    add("North", top);
    add("Center", rationalEnum);
    add("South", bottom);

    haltMenu = new Menu("Halt");
    haltMenu.add( haltItem = new MenuItem("Halt"));

    menubar = getMenuBar();
    menubar.add(haltMenu);
  } /* end setupScreen */


  public Dimension preferredSize() {
    return new Dimension(450, 410);
  }


  /* The guts of the thing. */

  public void run () {

    int i, j;

    NumDenom curNumDenom, tmpNumDenom;
    FpRep curFpRep     = new FpRep(-1), 
          curFpRepComp = new FpRep(-1);
    float curFloat;
    double plotNumbers[];

    setCursor(WAIT_CURSOR);
    curCursor.cursor = WAIT_CURSOR;

    int maxint = maxInt.getIntText();
    int minint = minInt.getIntText();

    if (minint >  maxint) {
      minint = maxint;
      minInt.setDoubleText(minint);
    }

    /* Put all the rational numbers in a list. */

    NumDenom numDenomList[] =
	new NumDenom[(maxint-minint+1)*(maxint-minint+1)];

    int size = 0;
    
    for (i = minint; i <= maxint; i++) {
      for (j = minint; j <= maxint; j++) {
	if (j != 0) numDenomList[size++] = new NumDenom(i,j);
      }
    }
    
    

    /* Sort the rational numbers.  Need to use a faster one here. */
    
    for (i = 0; i < size-1; i++) {
	for (j = i+1; j < size; j++) {
	    if (numDenomList[i].compare(numDenomList[j]) > 0) {
		tmpNumDenom = numDenomList[i];
		numDenomList[i] = numDenomList[j];
		numDenomList[j] = tmpNumDenom;
	    }
	}
    }

	  
    try {
	appThread.sleep(10);
    } catch (InterruptedException e) {}
    appThread.yield();
    

    /* Print the list out to the enumeration panel and create a list
       of floating-point numbers of plot.  Don't print duplicates. */
    
    int count = 0;
    rationalEnum.setText("");
    NumDenom cur, old = null;

    double xmin = numDenomList[0].toDouble();
    if (xmin > 0) xmin = -1;
    double xmax = numDenomList[size-1].toDouble();

    plotNumbers = new double[size];
    
    for (i = 0; i < size; i++) {

	cur = numDenomList[i];

	if (old != null && cur.compare(old) == 0) continue;

	if (old != null) rationalEnum.appendText(", ");
	old = cur;

	if (count%10 == 0 && count>0)
	  rationalEnum.appendText("\n");

	rationalEnum.appendText(cur.unparse());

	plotNumbers[count] = cur.toDouble();

	count++;

      }

    try {
	appThread.sleep(10);
    } catch (InterruptedException e) {}
    appThread.yield();

    plot.setXmin(xmin);
    peramXMin = xmin;
    plot.setXmax(xmax);
    plot.setArraySize(count);
    
    plot.setPlotPoints(plotNumbers);
    plot.repaint();

    sharedInfo.maxInt = maxint;
    sharedInfo.minInt = minint;
    sharedInfo.halted = false;
    eqn.parseIt();
    setCursor(defCursor);
    curCursor.cursor = defCursor;

  } /* End of run (doCalc) */


  public boolean action (Event e, Object arg) {
    int newFn;

    if (super.action(e, arg)) return(true);

      if ((e.target instanceof Button) 
		 && (e.id == Event.ACTION_EVENT)) {
	if (e.target == incrPlot) {
	  double xmax;

	  plot.setXmin(peramXMin);
	  plot.setXmax(plot.getXmax() * 2);
	  plot.repaint();
	  return true;
	}
	else if (e.target == decrPlot) {
	  double xmax;

	  plot.setXmax(plot.getXmax() / 2);
	  plot.repaint();
	  return true;
	}
      }
      else if (e.target instanceof ScrollableNumbers) {
	if (appThread != null)
	  appThread.stop();
	appThread = new Thread(this);
	appThread.start();
	plot.repaint();
	return true;
      }

      else if (e.target.equals( haltItem)) {
	if (appThread != null && appThread.isAlive())
	  sharedInfo.halted = true;
	stop();
	return true;
      }

  return false;
  }
} /* end of FP class */


class MyCanvas extends Panel
{

  /* May need to use threads to be sure canvas is repainted after init */

  double plotPoints[];
  int inset = 20;
  double ymax, ymin, xmax, xmin;
  int radius = 3;               // Size of the circles
  /* These are used in the axis */
  double xaxisPosy, yaxisPosx;
  int tickHalfLen = 2;
  PlotCoord pc;
  int arraySize;
  double tempMouseDown;
  /* for rubberbanding when selection new area */
  int xfrom;
  int oldTox, oldToy;
  int yfrom, yto;
  int xprev; 
  boolean prevSet;
  Frame enclosingFrame;
  Cursor curFrameCursor;

  MyCanvas(Frame enclosingFrameIn, Cursor curFrameCursorIn) {
    super();
    xmin = -100;
    xmax = 100;
    ymin = -2;
    ymax = 2;

    enclosingFrame = enclosingFrameIn; 
    curFrameCursor = curFrameCursorIn;
  }


  public Dimension preferredSize () {
    FontMetrics fm = getFontMetrics(getFont());
    return(new Dimension(500, 100));
  }


  public void setArraySize (int arraySizeIn)
  {
    arraySize = arraySizeIn;
  }


  public void setPlotPoints(double plotPointsIn[]) {
    plotPoints = plotPointsIn;
  }

  public void setXmin(double newXmin) {
    xmin = newXmin;
  }

  public double getXmin() {
    return(xmin);
  }

  public void setXmax(double newXmax) {
    xmax = newXmax;
  }

  public double getXmax() {
    return(xmax);
  }

  /*  PAINT
   */ 
  /** Draws a box around this panel, plus other stuff. */
  public void paint(Graphics g) {
    int x, y;
    Dimension d = size();
    /* These need to be made into floats. */
    double xcur, ycur = 1;
    int xcoord, ycoord;

    /* Draw a rectangle around the Canvas */
    g.drawRect(0,0, d.width - 1, d.height - 1);

    /* Set up the transformation between cartesian coordinates
       and screen coordinates. */
    pc = new PlotCoord (xmax, ymax, xmin, ymin, d.width, d.height, inset);

    /* If there is only one value for y, set up a little bit of room
       in the graph */
    if (ymax == ymin) {
      ymax = ymax + 2;
      ymin = ymin - 2;
    }

    drawAxis(g);

    /* In this case, we know we are always plotting at one */
    ycoord = pc.ycoord(1);

    int i, j;
    /*    System.out.println("arraySize: " + arraySize + 
		       "  arrayLength: " + plotPoints.length);*/
//    System.out.println("Xmin: " + xmin + "  Xmax: " + xmax);

    int xcoordPrev = -1;

/*    int coordat0 = pc.xcoord(0.0);
    System.out.println("Xcoord at 0 is " + coordat0);*/

    for (i = 0; i < arraySize; i++) {
	  
	xcur = plotPoints[i];

	if (xcur < xmin) {
	  continue;
	}
	if (xcur > xmax) {
	  /* nothing is on the graph, just exit */
	  if (i == 0)
	    return;

	  /* check to see if it really is larger - there seems
	     to be a slight problem in round-off */
	  if (xcur > (xmax + (xcur - plotPoints[i-1]) /100 ) ) {
	    /*System.out.println("We quit  on " + xcur + 
			       " and the previous value was " 
			       + plotPoints[i-1]);*/
	    break;
	  }
	}

	xcoord = pc.xcoord(xcur);
	
	/* don't draw the same line over and over again */
	if (i != 0) 
	  if (xcoord == xcoordPrev)
	    continue;

	g.drawLine(xcoord, ycoord-2, xcoord, ycoord+2);
	xcoordPrev = xcoord;

	/*	if (xcoord < coordat0)
	  System.out.println("xcoord is at < 0");*/

    }

  } /* end paint */

/*  Frame getFrame() {
    Class curClass;
    Container obj;
    
    curClass = getClass();
    obj = this;
    * System.out.println("Current class is " + curClass.getName()); *
    try {
    while (curClass != Class.forName("Rational.Rational") ) {
      obj = obj.getParent();
      curClass = obj.getClass();
      * System.out.println("Current class is " + curClass.getName()); *
      }
    } catch (ClassNotFoundException e) { 
      System.out.println("Class Not Found Exception");
    }
    return ((Frame) obj);
  }
*/


  /* This is for when you want to zero in on part of the graph */
  public boolean mouseDown(Event e, int xpos, int ypos) {

    enclosingFrame.setCursor(Frame.CROSSHAIR_CURSOR);

    yfrom = pc.ycoord(-2.5);
    yto   = pc.ycoord( 2.5);

    
    xfrom = xpos;
    tempMouseDown = pc.coordToX(xpos);
    if (tempMouseDown > xmax)
      tempMouseDown = xmax;
    return true;
  }

  public boolean mouseDrag(Event e, int xpos, int ypos) {
    Graphics g = getGraphics();
    int xto;

    xto = xpos;

    g.setXORMode(Color.pink);
    if (prevSet) {
      drawBox(xfrom, xprev, yfrom, yto, g);
    }
    drawBox(xfrom, xto, yfrom, yto, g);
    getToolkit().sync();
    xprev = xto;
    prevSet = true;
    g.setPaintMode();
    return true;
  }


  void drawBox(int xfrom, int xto, int yfrom, int yto, Graphics g) {
    g.drawLine(xfrom, yfrom, xfrom, yto);
    g.drawLine(xfrom, yto, xto, yto);
    g.drawLine(xto, yto, xto, yfrom);
    g.drawLine(xto, yfrom, xfrom, yfrom);
  }


  public boolean mouseUp(Event e, int xpos, int ypos) {
    Graphics g = getGraphics();
    double tempMouseUp;

    prevSet = false;
    tempMouseUp = pc.coordToX(xpos);
    if (tempMouseUp > xmax)
      tempMouseUp = xmax;
    
    /* If they just click in one spot, don't change anything */
    if (tempMouseDown == tempMouseUp)
      return true;

    if (tempMouseDown < tempMouseUp) {
      xmin = tempMouseDown;
      xmax = tempMouseUp;
    }
    else {
      xmin = tempMouseUp;
      xmax = tempMouseDown;
    }
      
    
    repaint();
    enclosingFrame.setCursor(curFrameCursor.cursor);
    return true;
  }

  /* DRAW AXIS
   */
  void drawAxis(Graphics g)
  {

    double xaxisStart = xmax, xaxisEnd = xmin, xaxisPosy;
    double yaxisStart = ymax, yaxisEnd = ymin, yaxisPosx;

    /* Axis rules:
       If 0 is on the map, print out the axis there.
       If 0 is not on the map, print out the axis on the 
       minimum. */

    if (xmin < 0  && xmax > 0) 
      yaxisPosx = 0;
    else
      yaxisPosx = xmin;

    if (ymin < 0  && ymax > 0) 
      xaxisPosy = 0;
    else
      xaxisPosy = ymin ;


    /* Draw x axis. */
    g.drawLine(pc.xcoord(xaxisStart), pc.ycoord(xaxisPosy), 
	       pc.xcoord(xaxisEnd), pc.ycoord(xaxisPosy));
    /* Draw y axis. */
    /*    g.drawLine(pc.xcoord(yaxisPosx), pc.ycoord(yaxisStart),
	       pc.xcoord(yaxisPosx), pc.ycoord(yaxisEnd));*/

    Font f = new Font ("Dialog", Font.PLAIN, 10);
    g.setFont(f);

    drawAllTicks(g);

  }


  void drawAllTicks(Graphics g) {
    double mult, multPart, dif, curTick, logNum = 0;
    int i = 0;
    double foo = 0;
    double lwr;

    /* draw a tick at 0 if it is on the graph */
    if (xmin <= 0  && xmax >= 0) {
      drawXTick(0, g);
    }

/*    if (ymin < 0  && ymax > 0) {
      drawYTick(0, g);
    }*/

    /* draw x ticks */
    dif = xmax - xmin;
    
    try {
      logNum = Math.log(dif)/Math.log(10);
    } catch (ArithmeticException e) {}

    logNum = Math.floor(logNum);

    /*System.out.println("xmax is " + xmax + ", xmin is " + xmin + 
                        ", i is " + i + ", logNum is " + logNum + ", dif is " + dif);*/

    if (dif/Math.pow(10,logNum) < 3) {
      foo = 2;
    }

    /*System.out.println("xmax is " + xmax + ", xmin is " + xmin + 
                        ", i is " + i + ", logNum is " + logNum + ", dif is " + dif); */

    if (foo == 0)
      mult = Math.pow(10, logNum);
    else
      mult = 2 * Math.pow(10, logNum-1);
    /*System.out.println("Mult is " + mult);*/

    lwr = xmin / mult;
    lwr = Math.ceil(lwr);
    lwr = lwr * mult;

    for ( curTick = lwr; curTick <= xmax; 
	                        curTick += mult) {
       drawXTick(curTick, g);
     }
    /*    for ( curTick = 0 - mult; curTick >= xmin; 
	                        curTick -= mult) {
       drawXTick(curTick, g);
     }*/

    /* draw y ticks */
    /*    drawYTick(2.0, g);*/

  }

  void drawYTick(double y, Graphics g) {
    int xcoord, ycoord;
    int offset;

    ycoord = pc.ycoord(y);
    xcoord = pc.xcoord(yaxisPosx);
    
    if (y == 0)
      offset = 10;
    else
      offset = 5;

    g.drawLine(xcoord + tickHalfLen, 
	       ycoord , xcoord - tickHalfLen, ycoord);
    g.drawString(new Float(y).toString(),
		 xcoord + 5, ycoord + offset);

/*    g.drawLine(xcoord + tickHalfLen, ycoord , xcoord - tickHalfLen, ycoord);
    g.drawString(new Float(y).toString(),
		 xcoord + 5, ycoord + 10);*/
  }

  void drawXTick(double x, Graphics g) {
    int xcoord, ycoord;

    xcoord = pc.xcoord(x);
    ycoord = pc.ycoord(xaxisPosy);

    g.drawLine(xcoord, ycoord + tickHalfLen, 
	       xcoord , ycoord - tickHalfLen);
    if (x == 0)
       g.drawString(new Float(x).toString(),
		   xcoord, ycoord + 15);
    else
       g.drawString(new Float(x).toString(),
		   xcoord - 5, ycoord + 15);
      

  }

} /* End MyPlot */



/* CLASS        PLOT COORD
 *
 * 
 */
class PlotCoord {
  
  double xstretch = 1;
  double ystretch = 1;
  double xshift = 0;
  double yshift = 0;
  double ywholeScreen;       
  int inset;             /* How far from the edge the display is */

  PlotCoord(double xmax, double ymax, double xmin, double ymin, 
	    int width, int height, int insetin ) {

    inset = insetin;

    xstretch = (width - (2 * inset))/ (xmax - xmin);
    ystretch = (height - (2 * inset)) / (ymax - ymin);

    xshift = -xmin;
    yshift = -ymin;

    ywholeScreen = ymax - ymin;

  }

  /* Return the X and Y coordinates on the canvas. */
  int xcoord(double x) {
    double xcoordFloat;
    
    xcoordFloat = (x + xshift) * xstretch + inset;
    return(new Long(Math.round(xcoordFloat)).intValue());
  }

  int ycoord(double y) {
    double ycoordFloat;
    
    ycoordFloat = (ywholeScreen - (y + yshift)) * ystretch + inset;
    return(new Long(Math.round(ycoordFloat)).intValue());
  }
       
  /* Return the X and Y real coords on canvas. */
  double coordToX(int xcoord) {
    double x;

    x = (xcoord - inset) / xstretch - xshift;
    return(x);
  }

  double coordToY(int ycoord) {
    double y;

    y = ((ycoord - inset) / ystretch) + ywholeScreen - yshift;
    return(y);
  }
       

}

class NumDenom extends Object {
	int num;
	int denom;

    NumDenom (int numIn, int denomIn) {
	num = numIn;
	denom = denomIn;
	if (denom < 0) {
	  num = -num;
	  denom = -denom;
	}
	if (num == 0) {
	    denom = 1;
	}
	int gcd = GCD.gcd(num, denom);
	num = num/gcd;
	denom = denom/gcd;
    }

    int compare (NumDenom other) {
	int p1 = num*other.denom;
	int p2 = denom*other.num;
	if (p1 > p2) return(1);
	if (p2 > p1) return(-1);
	return(0);
    }

    double toDouble () {
	return((1.0*num) / denom);
    }
    

 String unparse () {
     if (denom == 1)
	 return(String.valueOf(num));
     return(String.valueOf(num) + "/" + String.valueOf(denom));
 }
		
}

class FpRep extends Object {
	float fp;

  public FpRep (float fpIn)
  {
    fp = fpIn;
  }

}

class Cursor {
  int cursor;
}

class GCD {

  /* figure out the greatest common denominator */
  public static int gcd(int u, int v) {
    int r;

    u = Math.abs(u);
    v = Math.abs(v);

    while (true) {
      if (v == 0)
	return u;
      r = u % v;
      u = v;
      v = r;
    }

  }

}

class RatSharedInfo {
  int minInt,
      maxInt;

  boolean halted = false;

}
